@@ -4,7 +4,7 @@
 #
 
 package Parallel::WorkUnit;
-$Parallel::WorkUnit::VERSION = '1.012';
+$Parallel::WorkUnit::VERSION = '1.100';
 use v5.8;
 
 # ABSTRACT: Provide easy-to-use forking with ability to pass back data
@@ -14,14 +14,27 @@ use warnings;
 use autodie;
 
 use Try::Tiny;
-my $do_thread;
+
+#
+# Setting up threads, on Win32, if appropriate
+#
+
+my $use_anyevent_pipe;
 ## no critic (BuiltinFunctions::ProhibitStringyEval)
-$do_thread = eval 'use threads qw//; 1' if $^O eq 'MSWin32';
+$use_anyevent_pipe = eval 'use AnyEvent::Util qw//; 1' if $^O eq 'MSWin32';
 ## critic
-if ($do_thread) { eval 'use Thread::Queue;'; }
+
+my $use_threads;
+## no critic (BuiltinFunctions::ProhibitStringyEval)
+$use_threads = eval 'use threads qw//; 1' if ( ( $^O eq 'MSWin32' ) && ( !$use_anyevent_pipe ) );
+## critic
+if ($use_threads) { eval 'use Thread::Queue;'; }
+
+my $use_thread_queue = ( $use_threads && ( !$use_anyevent_pipe ) );
 
 use Carp;
 
+use IO::Handle;
 use IO::Pipe;
 use IO::Select;
 use Moose;
@@ -37,6 +50,23 @@ subtype 'Parallel::WorkUnit::PositiveInt', as 'Int',
   message { "The number you provided, $_, was not a positive number" };
 
 
+has use_anyevent => (
+    is      => 'rw',
+    isa     => 'Bool',
+    trigger => \&_set_anyevent,
+);
+
+has _cv => (
+    is  => 'rw',
+    isa => 'Maybe[AnyEvent::CondVar]',
+);
+
+has _last_error => (
+    is  => 'rw',
+    isa => 'Maybe[Str]',
+);
+
+
 has 'max_children' => (
     is      => 'rw',
     isa     => 'Maybe[Parallel::WorkUnit::PositiveInt]',
@@ -78,7 +108,7 @@ sub BUILD {
 
     $self->_subprocs( {} );
 
-    if ($do_thread) {
+    if ($use_thread_queue) {
         $self->_queue( Thread::Queue->new() );
     }
 
@@ -90,10 +120,19 @@ sub async {
     if ( $#_ != 2 ) { confess 'invalid call'; }
     my ( $self, $sub, $callback ) = @_;
 
-    my $pipe = IO::Pipe->new();
+    # If there are pending errors, throw that.
+    if ( defined( $self->_last_error ) ) { die( $self->_last_error ); }
+
+    my $pipe;
+    if ($use_anyevent_pipe) {
+        $pipe = [];
+        (@$pipe) = AnyEvent::Util::portable_pipe();
+    } else {
+        $pipe = IO::Pipe->new();
+    }
 
     my ( $pid, $thr );
-    if ($do_thread) {
+    if ($use_threads) {
         $pid = $self->_count();
         $self->_count( $pid + 1 );
 
@@ -105,15 +144,25 @@ sub async {
 
     if ($pid) {
         # We are in the parent process
-        $pipe->reader();
+        if ($use_anyevent_pipe) {
+            $pipe = $pipe->[0];
+        } else {
+            $pipe->reader();
+        }
 
         $self->_subprocs()->{$pid} = {
             fh       => $pipe,
+            anyevent => undef,
             callback => $callback,
             caller   => [ caller() ],
             thread   => $thr
         };
 
+        # Set up anyevent listener if appropriate
+        if ( $self->use_anyevent() ) {
+            $self->_add_anyevent_watcher($pid);
+        }
+
         return $pid;
 
     } else {
@@ -126,7 +175,11 @@ sub _child {
     my ( $self, $sub, $pipe, $pid ) = @_;
 
     # We are in the child process
-    $pipe->writer();
+    if ($use_anyevent_pipe) {
+        $pipe = $pipe->[1];
+    } else {
+        $pipe->writer();
+    }
     $pipe->autoflush(1);
 
     try {
@@ -138,7 +191,7 @@ sub _child {
     };
 
     # Windows doesn't do fork(), it does threads...
-    if ($do_thread) {
+    if ($use_threads) {
         return 1;
     } else {
         exit();
@@ -150,8 +203,31 @@ sub waitall {
     if ( $#_ != 0 ) { confess 'invalid call'; }
     my ($self) = @_;
 
+    # No subprocs?  Just return.
+    if ( scalar( keys %{ $self->_subprocs } ) == 0 ) {
+        if ( $self->use_anyevent ) {
+            $self->_cv( AnyEvent->condvar );
+        }
+
+        return;
+    }
+
+    # Using AnyEvent?
+    if ( defined( $self->_cv ) ) {
+        $self->_cv->recv();
+        if ( defined( $self->_last_error ) ) {
+            my $err = $self->_last_error;
+            $self->_last_error(undef);
+
+            die($err);
+        }
+
+        return;
+    }
+
     # Tail recursion
-    if ( $self->waitone() ) { goto &waitall }
+    if ( $self->_waitone() ) { goto &waitall }
+    return;
 }
 
 
@@ -159,10 +235,29 @@ sub waitone {
     if ( $#_ != 0 ) { confess 'invalid call'; }
     my ($self) = @_;
 
+    my $rv = $self->_waitone();
+
+    # Using AnyEvent?
+    if ( defined( $self->_last_error ) ) {
+        my $err = $self->_last_error;
+        $self->_last_error(undef);
+
+        die($err);
+    }
+
+    return $rv;
+}
+
+# Meat of waitone (but doesn't handle returning an exception when using
+# anyevent)
+sub _waitone {
+    if ( $#_ != 0 ) { confess 'invalid call'; }
+    my ($self) = @_;
+
     my $sp = $self->_subprocs();
     if ( !keys(%$sp) ) { return; }
 
-    if ($do_thread) {
+    if ($use_thread_queue) {
         # On Windows
         #
         my $child = $self->_queue()->dequeue();
@@ -190,7 +285,7 @@ sub waitone {
                         my $thr = $self->_subprocs()->{$child}{thread};
                         $self->_read_result($child);
 
-                        if ($do_thread) {
+                        if ($use_threads) {
                             $thr->join();
                         } else {
                             waitpid( $child, 0 );
@@ -216,6 +311,23 @@ sub wait {
     if ( $#_ != 1 ) { confess 'invalid call'; }
     my ( $self, $pid ) = @_;
 
+    my $rv = $self->_wait($pid);
+
+    if ( defined( $self->_last_error ) ) {
+        my $err = $self->_last_error;
+        $self->_last_error(undef);
+
+        die($err);
+    }
+
+    return $rv;
+}
+
+# Internal version that doesn't check for AnyEvent die needs
+sub _wait {
+    if ( $#_ != 1 ) { confess 'invalid call'; }
+    my ( $self, $pid ) = @_;
+
     if ( !exists( $self->_subprocs()->{$pid} ) ) {
 
         # We don't warn/die because it's possible that there is
@@ -226,7 +338,7 @@ sub wait {
     my $thr    = $self->_subprocs()->{$pid}{thread};
     my $result = $self->_read_result($pid);
 
-    if ($do_thread) {
+    if ($use_threads) {
         $thr->join();
     } else {
         waitpid( $pid, 0 );
@@ -250,6 +362,9 @@ sub queue {
     if ( $#_ != 2 ) { confess 'invalid call'; }
     my ( $self, $sub, $callback ) = @_;
 
+    # If there are pending errors, throw that.
+    if ( defined( $self->_last_error ) ) { die( $self->_last_error ); }
+
     push @{ $self->_queued_children }, [ $sub, $callback ];
     return $self->_start_queued_children();
 }
@@ -278,7 +393,7 @@ sub _send {
         die 'freeze() returned undef for child return value';
     }
 
-    if ($do_thread) {
+    if ($use_thread_queue) {
         $self->_queue()->enqueue($pid);
     }
 
@@ -333,12 +448,22 @@ sub _read_result {
     if ( $type eq 'RESULT' ) {
         $cinfo->{callback}->($data);
     } else {
-        if ($do_thread) { $thr->join(); }
-        die(    "Child (created at "
-              . $caller->[1]
-              . " line "
-              . $caller->[2]
-              . ") died with error: $data" );
+        if ($use_threads) { $thr->join(); }
+
+        my $err =
+            "Child (created at "
+          . $caller->[1]
+          . " line "
+          . $caller->[2]
+          . ") died with error: $data";
+
+        if ( $self->use_anyevent ) {
+            # Can't throw events with anyevent
+            $self->_last_error($err);
+        } else {
+            # Otherwise we do throw it
+            die($err);
+        }
     }
 
     return;
@@ -351,6 +476,7 @@ sub _start_queued_children() {
     my ($self) = @_;
 
     if ( !( @{ $self->_queued_children } ) ) { return; }
+    if ( defined( $self->_last_error ) )     { return; }    # Do not queue if there are errors
 
     # Can we start a queued process?
     while ( scalar @{ $self->_queued_children } ) {
@@ -368,6 +494,68 @@ sub _start_queued_children() {
     return 1;
 }
 
+# Sets up AnyEvent or tears it down as needed
+sub _set_anyevent() {
+    if ( $#_ < 1 ) { confess 'invalid call' }
+    if ( $#_ > 2 ) { confess 'invalid call' }
+    my ( $self, $new, $old ) = @_;
+
+    if ( ( !$old ) && $new ) {
+        # We are setting up AnyEvent
+        require AnyEvent;
+
+        if ( defined( $self->_subprocs() ) ) {
+            foreach my $pid ( keys %{ $self->_subprocs() } ) {
+                my $proc = $self->_subprocs()->{$pid};
+
+                $self->_add_anyevent_watcher($pid);
+            }
+        }
+
+        $self->_cv( AnyEvent->condvar );
+
+    } elsif ( $old && ( !$new ) ) {
+        # We are tearing down AnyEvent
+
+        if ( defined( $self->_subprocs() ) ) {
+            foreach my $pid ( keys %{ $self->_subprocs() } ) {
+                my $proc = $self->_subprocs()->{$pid};
+
+                $proc->{anyevent} = undef;
+            }
+        }
+
+        $self->_cv(undef);
+    }
+    return;
+}
+
+# Sets up the listener for AnyEvent
+sub _add_anyevent_watcher() {
+    if ( $#_ != 1 ) { confess 'invalid call' }
+    my ( $self, $pid ) = @_;
+
+    my $proc = $self->_subprocs()->{$pid};
+
+    $proc->{anyevent} = AnyEvent->io(
+        fh   => $proc->{fh},
+        poll => 'r',
+        cb   => sub {
+            $self->_wait($pid);
+            if ( scalar( keys %{ $self->_subprocs() } ) == 0 ) {
+                my $oldcv = $self->_cv;
+                $self->_cv( AnyEvent->condvar );
+                $oldcv->send();
+            }
+
+            # Start queued children, if needed
+            $self->_start_queued_children();
+        },
+    );
+
+    return;
+}
+
 __PACKAGE__->meta->make_immutable;
 
 1;
@@ -384,10 +572,13 @@ Parallel::WorkUnit - Provide easy-to-use forking with ability to pass back data
 
 =head1 VERSION
 
-version 1.012
+version 1.100
 
 =head1 SYNOPSIS
 
+  #
+  # Standard Interface
+  #
   my $wu = Parallel::WorkUnit->new();
   $wu->async( sub { ... }, \&callback );
 
@@ -397,6 +588,16 @@ version 1.012
   $wu->queue( sub { ... }, \&callback );
   $wu->waitall();
 
+
+  #
+  # AnyEvent Interface
+  #
+  use AnyEvent;
+
+  $wu->use_anyevent(1);
+  $wu->async( sub { ... }, \&callback );
+  $wu->waitall();  # Not strictly necessary
+
 =head1 DESCRIPTION
 
 This is a very simple forking implementation of parallelism, with the
@@ -406,11 +607,25 @@ the information, serialized, back).  It was designed to be very simple
 for a developer to use, with the ability to pass reasonably large amounts
 of data back to the parent process.
 
+This module is also designed to work with AnyEvent when desired.
+
 There are many other Parallel::* applications in CPAN - it would be worth
 any developer's time to look through those and choose the best one.
 
 =head1 ATTRIBUTES
 
+=head2 use_anyevent
+
+  $wu->use_anyevent(1);
+
+If set to a value that is true, creates AnyEvent watchers for each
+asyncronous or queued job.  The equivilent of an C<AnyEvent> condition
+variable C<recv()>, used when all processes finish executing, is the
+C<waitall()> method.  However, the processes are integrated into a
+standard C<AnyEvent> loop, so it isn't strictly necessary to callC<waitall()>.
+In addition, a call to C<waitall()> will execute other processes in
+the C<AnyEvent> event loop.
+
 =head2 max_children
 
   $wu->max_children(5);
@@ -420,7 +635,13 @@ any developer's time to look through those and choose the best one.
 
 If set to a value other than undef, limits the number of outstanding
 queue children (created by the C<queue()> method) that can be executing
-at any given time.  This defaults to 5.
+at any given time.
+
+This defaults to 5.
+
+This attribute does not impact the C<async()> method's ability to
+create children, but these children will count against the limit used
+by C<queue()>.
 
 Calling without any parameters will return the number of children.
 
@@ -463,10 +684,10 @@ can spawn a new child regardless of the number of children already
 spawned. However, you children started with this method still count
 against the limit used by C<queue()>.
 
-Note: on Windows with threaded Perl, threads instead of forks are used.
-See C<thread> for the caveats that apply.  The PID returned is instead
-a meaningless (outside of this module) counter, not associated with any
-Windows thread identifier.
+Note: on Windows with threaded Perl, if C<AnyEvent> is not installed,
+threads instead of forks are used.  See C<thread> for the caveats
+that apply.  The PID returned is instead a meaningless (outside of
+this module) counter, not associated with any Windows thread identifier.
 
 =head2 waitall()
 
@@ -517,16 +738,6 @@ C<max_children>) and not returning a PID.  Instead, a value of 1
 is returned if the process is immediately started, C<undef>
 otherwise.
 
-=head1 BUGS
-
-Windows doesn't do C<fork()>, but emulates it with threads.  As a result,
-any thread unsafe library is going to cause problems with Windows.  In
-addition, all the normal thread caveats apply - see L<threads> for more
-information.
-
-In addition, this code is unlikely to function properly on a Windows without
-threaded Perl.
-
 =head1 AUTHOR
 
 Joelle Maslak <jmaslak@antelope.net>
